;;; irc-command.el
;;;
;;; copyright (C) 1996 lars, brinkhoff
;;;
;;; written 961011-961229

(defvar irc-command-function-alist nil)

(if (not irc-command-function-alist)
    (setq irc-command-function-alist
	  '(("" . irc-command-say)
	    ("away" . irc-command-away)
	    ("ctcp" . irc-command-ctcp)
	    ("dcc" . irc-command-dcc)
	    ("exit" . irc-command-exit)
	    ("join" . irc-command-join)
	    ("kick" . irc-command-kick)
	    ("leave" . irc-command-part)
	    ("list" . irc-command-list)
	    ("me" . irc-command-me)
	    ("msg" . irc-command-privmsg)
	    ("names" . irc-command-names)
	    ("nick" . irc-command-nick)
	    ("part" . irc-command-part)
	    ("ping" . irc-command-ping)
	    ("privmsg" . irc-command-privmsg)
	    ("say" . irc-command-say)
	    ("server" . irc-command-server)
	    ("topic" . irc-command-topic)
	    ("who" . irc-command-who)
	    ("whois" . irc-command-whois)
	    ("window" . irc-command-window)
	    ("whowas" . irc-command-whowas)
	    (nil . irc-command-say)
	    )))

(defun irc-command-list (args)
  (irc-send-list))

(defun irc-command-kick (args)
  (let ((argc (length args)))
    (cond
     ((= argc 2)
      (irc-send-kick (irc-current-channel) (irc-arg 0 args)))
     ((= argc 3)
      (irc-send-kick (irc-arg 0 args) (irc-arg 1 args)))
     (t
      (irc-printf "*** usage: kick [channel] nick")))))

(defun irc-command-names (args)
  (let ((string (irc-arg 0 args)))
    (if string
	(irc-send-names (irc-arg 0 args))
      (irc-send-names (irc-current-channel)))))

(defun irc-command-me (args)
  (let ((message (irc-rest 0 args)))
    (irc-command-ctcp (list (irc-current-channel) "ACTION" message))))

(defun irc-command-say (args)
  (let ((message (irc-rest 0 args)))
    (interactive "sMessage: ")
    (irc-command-privmsg (list (irc-current-channel) message))))

(defun irc-command (command &optional string)
  (let ((args (irc-split string))
	(f (cdr (assoc (if command (downcase command) nil)
		       irc-command-function-alist))))
    (if f
	(funcall f args)
      (irc-command-unknown (list command)))))

(defun irc-command-nick (args)
  (let ((nick (irc-arg 0 args)))
    (irc-send-nick nick)))

(defun irc-command-join (args)
  (let ((channel (irc-arg 0 args)))
    (if channel
	(progn
	  (irc-send-part (irc-current-channel))
	  (irc-send-join channel)))))

(defun irc-command-part (args)
  (let ((channel (irc-arg 0 args)))
    (if channel
	(irc-send-part channel)
      (if (irc-current-channel)
	  (irc-send-part (irc-current-channel))))))

(defun irc-comand-me (args)
  (let ((message (irc-rest 0 args)))
    (irc-print-me message)
    (irc-send-me message)))

(defun irc-command-who (args)
  (let ((string (irc-rest 0 args)))
    (irc-sendf "WHO :%s" string)))

(defun irc-command-whois (args)
  (let ((string (irc-rest 0 args)))
    (irc-sendf "WHOIS :%s" string)))

(defun irc-command-whowas (args)
  (let ((string (irc-rest 0 args)))
    (irc-sendf "WHOWAS :%s" string)))

(defun irc-command-topic (args)
  (let ((string (irc-rest 0 args)))
    (if string
	(irc-sendf "TOPIC %s :%s" (irc-current-channel) string)
      (irc-sendf "TOPIC %s" (irc-current-channel)))))

(defun irc-command-unknown (args)
  (let ((command (irc-arg 0 args)))
    (irc-printf "*** Unknown command: %s [client]" command)))

(defun irc-command-away (args)
  (let ((message (irc-rest 0 args)))
    (irc-send-away message)))

(defun irc-kill-stream (stream message)
  (let ((list (assoc stream irc-stream-alist)))
    (if list
	(let* ((p (car list))
	       (live (and p (processp p) (eq (process-status p) 'open)))
	       (buffer (if live (process-buffer p))))
	  (if live
	      (progn
		(if (buffer-live-p buffer)
		    (progn
		      (save-excursion
			(set-buffer buffer)
			(irc-send-quit message)
			(delete-windows-on buffer)
			(kill-buffer buffer))))
		(delete-process p)))))))

(defun irc-kill-stream-buffers (stream)
  (let ((list (assoc stream irc-stream-alist)))
    (if list
	(let ((i (cdr list)))
	  (while i
	    (if (buffer-live-p (cdr (car i)))
		(save-excursion
		  (set-buffer (cdr (car i)))
		  (irc-command-window-kill)))
	    (setq i (cdr i)))))))

(defun irc-command-exit (args)
  (let ((message (irc-rest 0 args))
	(i irc-stream-alist))
    (while i
      (irc-kill-stream (car (car i)) message)
      (irc-kill-stream-buffers (car (car i)))
      (setq i (cdr i))))
  (setq irc-stream-alist nil
	irc-stream-nick-alist nil))

(defun irc-command-privmsg (args)
  (let ((to (irc-arg 0 args))
	(message (irc-rest 1 args)))
    (if (> (length message) 0)
	(progn
	  (irc-send-privmsg to message)
	  (irc-print-privmsg irc-nick to message t)))))

(defun irc-command-dcc (args)
  (let ((command (irc-arg 0 args)))
    (irc-printf "*** Note: DCC commands aren't implemented yet")
    (if command
	(setq command (downcase command)))
    (cond
     ((eq command nil)
      (irc-printf "Type    Nick      Status     Start time           Sent     Read     Arguments"))
     ((string= command "send")
      nil)
     ((string= command "close")
      nil)
     ((string= command "get")
      nil)
     (t
      (irc-printf "*** Unknown DCC command: %s" (upcase command))))))

(defun irc-command-ping (args)
  (irc-command-ctcp (list (irc-arg 0 args) "ping")))

(defun irc-command-ctcp (args)
  (irc-print-ctcp irc-nick (irc-arg 1 args) (irc-rest 2 args))
  (let ((to (irc-arg 0 args))
	(command (irc-arg 1 args))
	(arg (irc-rest 2 args)))
    (if command
	(irc-send-ctcp to command arg)
      (irc-send-ctcp to "version"))))

(defun irc-command-server (args)
  (interactive "sHost: ")
  (let ((host (irc-arg 0 args))
	(port (irc-arg 1 args)))
    (if (not port)
	(setq port 6667)
      (if (stringp port)
	  (setq port (string-to-int port))))
    (irc-printf "*** Connecting to port %d of server %s" port host)
    (let ((stream-nick (irc-open-server host port irc-nick)))
      (if stream-nick
	  (progn
	    (irc-set-buffer-stream (current-buffer) (car stream-nick))
	    (irc-set-stream-nick (car stream-nick) (nth 1 stream-nick)))))))

(defun irc-command-window (args)
  (let ((command (irc-arg 0 args))
	(string (irc-rest 1 args)))
    (cond
     ((string= command "new")
      (irc-command-window-new))
     ((string= command "kill")
      (irc-command-window-kill))
     (t
      (irc-printf "*** Unknown WINDOW command: %s" command)))))

(defun irc-command-window-new ()
  (interactive)
  (irc-open-window (list irc-stream irc-nick)))

(defun irc-command-window-kill ()
  (interactive)
  (irc-command "part" (irc-current-channel))
  (let ((buffer (current-buffer)))
    (message (format "irc-stream-alist(old): %s" irc-stream-alist))
    (sit-for 2)
    (irc-delete-buffer buffer)
    (message (format "irc-stream-alist(new): %s" irc-stream-alist))
    (sit-for 2)
    (delete-windows-on buffer)
    (kill-buffer buffer)))
  
;;; end of irc-command.el
